@@ -23,17 +23,31 @@
         old <- getAutoBPPARAM()
         setAutoBPPARAM(BPPARAM)
         on.exit(setAutoBPPARAM(old))
-        discard <- rowAnyNAs(DelayedArray(x))
 
+        y <- DelayedArray(x)
+        discard <- rowAnyNAs(y)
         if (any(discard)) {
-            warning(sprintf("'%s' contains rows with missing values", msg))
-            x <- x[!discard,,drop=FALSE]
+            # Returning a DelayedArray to avoid making an actual subset.
+            x <- y[!discard,,drop=FALSE]
         }
     }
 
     x
 }
 
+.create_intersection <- function(test, reference) {
+    # Effectively an NA-safe intersect() that preserves ordering in 'test'.
+    common <- test[test %in% reference]
+    common <- common[!is.na(common)]
+    common <- common[!duplicated(common)]
+
+    # match() takes the first occurrence, consistent with internal behavior in singlepp.
+    list(
+        test = match(common, test),
+        reference = match(common, reference)
+    )
+}
+
 #' @importFrom methods is
 #' @importClassesFrom S4Vectors List
 .is_list <- function(val) {
@@ -83,3 +97,5 @@
         as.matrix(x)
     }
 }
+
+utils::globalVariables(".data")
